@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SidplayDecoderPlugin.hxx"
 #include "decoder/Features.h"
@@ -23,23 +7,19 @@
 #include "tag/Handler.hxx"
 #include "tag/Builder.hxx"
 #include "song/DetachedSong.hxx"
+#include "fs/NarrowPath.hxx"
 #include "fs/Path.hxx"
 #include "fs/AllocatedPath.hxx"
+#include "lib/fmt/PathFormatter.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "lib/icu/Converter.hxx"
-#ifdef HAVE_SIDPLAYFP
-#include "fs/io/FileReader.hxx"
-#include "util/RuntimeError.hxx"
-#endif
-#include "util/StringFormat.hxx"
-#include "util/StringView.hxx"
+#include "io/FileReader.hxx"
 #include "util/Domain.hxx"
 #include "util/AllocatedString.hxx"
 #include "util/CharUtil.hxx"
 #include "util/ByteOrder.hxx"
-#include "util/RuntimeError.hxx"
 #include "Log.hxx"
 
-#ifdef HAVE_SIDPLAYFP
 #include <sidplayfp/sidplayfp.h>
 #include <sidplayfp/SidInfo.h>
 #include <sidplayfp/SidConfig.h>
@@ -48,12 +28,8 @@
 #include <sidplayfp/builders/resid.h>
 #include <sidplayfp/builders/residfp.h>
 #include <sidplayfp/SidDatabase.h>
-#else
-#include <sidplay/sidplay2.h>
-#include <sidplay/builders/resid.h>
-#include <sidplay/utils/SidTuneMod.h>
-#include <sidplay/utils/SidDatabase.h>
-#endif
+
+#include <fmt/format.h>
 
 #include <iterator>
 #include <memory>
@@ -73,28 +49,21 @@ struct SidplayGlobal {
 
 	bool filter_setting;
 
-#ifdef HAVE_SIDPLAYFP
 	std::unique_ptr<uint8_t[]> kernal, basic;
-#endif
 
 	explicit SidplayGlobal(const ConfigBlock &block);
 };
 
 static SidplayGlobal *sidplay_global;
 
-#ifdef HAVE_SIDPLAYFP
 static constexpr unsigned rom_size = 8192;
 
 static void loadRom(const Path rom_path, uint8_t *dump)
 {
 	FileReader romDump(rom_path);
-	if (romDump.Read(dump, rom_size) != rom_size)
-	{
-		throw FormatRuntimeError
-			("Could not load rom dump '%s'", rom_path.c_str());
-	}
+	if (romDump.Read(std::as_writable_bytes(std::span{dump, rom_size})) != rom_size)
+		throw FmtRuntimeError("Could not load rom dump {:?}", rom_path);
 }
-#endif
 
 /**
  * Throws on error.
@@ -103,14 +72,10 @@ static std::unique_ptr<SidDatabase>
 sidplay_load_songlength_db(const Path path)
 {
 	auto db = std::make_unique<SidDatabase>();
-#ifdef HAVE_SIDPLAYFP
 	bool error = !db->open(path.c_str());
-#else
-	bool error = db->open(path.c_str()) < 0;
-#endif
 	if (error)
-		throw FormatRuntimeError("unable to read songlengths file %s: %s",
-					 path.c_str(), db->error());
+		throw FmtRuntimeError("unable to read songlengths file {:?}: {}",
+				      path, db->error());
 
 	return db;
 }
@@ -132,12 +97,11 @@ SidplayGlobal::SidplayGlobal(const ConfigBlock &block)
 
 	filter_setting = block.GetBlockValue("filter", true);
 
-#ifdef HAVE_SIDPLAYFP
 	/* read kernal rom dump file */
 	const auto kernal_path = block.GetPath("kernal");
 	if (!kernal_path.IsNull())
 	{
-		kernal = std::make_unique<uint8_t[]>(rom_size);
+		kernal = std::make_unique_for_overwrite<uint8_t[]>(rom_size);
 		loadRom(kernal_path, kernal.get());
 	}
 
@@ -145,10 +109,9 @@ SidplayGlobal::SidplayGlobal(const ConfigBlock &block)
 	const auto basic_path = block.GetPath("basic");
 	if (!basic_path.IsNull())
 	{
-		basic = std::make_unique<uint8_t[]>(rom_size);
+		basic = std::make_unique_for_overwrite<uint8_t[]>(rom_size);
 		loadRom(basic_path, basic.get());
 	}
-#endif
 }
 
 static bool
@@ -169,7 +132,7 @@ struct SidplayContainerPath {
 	unsigned track;
 };
 
-gcc_pure
+[[gnu::pure]]
 static unsigned
 ParseSubtuneName(const char *base) noexcept
 {
@@ -193,10 +156,9 @@ ParseSubtuneName(const char *base) noexcept
 static SidplayContainerPath
 ParseContainerPath(Path path_fs) noexcept
 {
-	const Path base = path_fs.GetBase();
+	const NarrowPath base = NarrowPath(path_fs.GetBase());
 	unsigned track;
-	if (base.IsNull() ||
-	    (track = ParseSubtuneName(base.c_str())) < 1)
+	if (!base || (track = ParseSubtuneName(base)) < 1)
 		return { AllocatedPath(path_fs), 1 };
 
 	return { path_fs.GetDirectoryName(), track };
@@ -243,17 +205,10 @@ sidplay_file_decode(DecoderClient &client, Path path_fs)
 	/* load the tune */
 
 	const auto container = ParseContainerPath(path_fs);
-#ifdef HAVE_SIDPLAYFP
-	SidTune tune(container.path.c_str());
-#else
-	SidTuneMod tune(container.path.c_str());
-#endif
+	auto np = NarrowPath(container.path);
+	auto tune = SidTune(np);
 	if (!tune.getStatus()) {
-#ifdef HAVE_SIDPLAYFP
 		const char *error = tune.statusString();
-#else
-		const char *error = tune.getInfo().statusString;
-#endif
 		FmtWarning(sidplay_domain, "failed to load file: {}", error);
 		return;
 	}
@@ -267,21 +222,12 @@ sidplay_file_decode(DecoderClient &client, Path path_fs)
 
 	/* initialize the player */
 
-#ifdef HAVE_SIDPLAYFP
 	sidplayfp player;
 
 	player.setRoms(sidplay_global->kernal.get(),
 		       sidplay_global->basic.get(),
 		       nullptr);
-#else
-	sidplay2 player;
-#endif
-#ifdef HAVE_SIDPLAYFP
-	bool error = !player.load(&tune);
-#else
-	bool error = player.load(&tune) < 0;
-#endif
-	if (error) {
+	if (!player.load(&tune)) {
 		FmtWarning(sidplay_domain,
 			   "sidplay2.load() failed: {}", player.error());
 		return;
@@ -289,7 +235,6 @@ sidplay_file_decode(DecoderClient &client, Path path_fs)
 
 	/* initialize the builder */
 
-#ifdef HAVE_SIDPLAYFP
 	ReSIDfpBuilder builder("ReSID");
 	if (!builder.getStatus()) {
 		FmtWarning(sidplay_domain,
@@ -305,88 +250,33 @@ sidplay_file_decode(DecoderClient &client, Path path_fs)
 			   builder.error());
 		return;
 	}
-#else
-	ReSIDBuilder builder("ReSID");
-	builder.create(player.info().maxsids);
-	if (!builder) {
-		FmtWarning(sidplay_domain, "ReSIDBuilder.create() failed: {}",
-			   builder.error());
-		return;
-	}
-#endif
 
 	builder.filter(sidplay_global->filter_setting);
-#ifdef HAVE_SIDPLAYFP
 	if (!builder.getStatus()) {
 		FmtWarning(sidplay_domain,
 			   "ReSIDfpBuilder.filter() failed: {}",
 			   builder.error());
 		return;
 	}
-#else
-	if (!builder) {
-		FmtWarning(sidplay_domain, "ReSIDBuilder.filter() failed: {}",
-			   builder.error());
-		return;
-	}
-#endif
 
 	/* configure the player */
 
 	auto config = player.config();
 
-#ifndef HAVE_SIDPLAYFP
-	config.clockDefault = SID2_CLOCK_PAL;
-	config.clockForced = true;
-	config.clockSpeed = SID2_CLOCK_CORRECT;
-#endif
 	config.frequency = 48000;
-#ifndef HAVE_SIDPLAYFP
-	config.optimisation = SID2_DEFAULT_OPTIMISATION;
-
-	config.precision = 16;
-	config.sidDefault = SID2_MOS6581;
-#endif
 	config.sidEmulation = &builder;
-#ifdef HAVE_SIDPLAYFP
 	config.samplingMethod = SidConfig::INTERPOLATE;
 	config.fastSampling = false;
-#else
-	config.sidModel = SID2_MODEL_CORRECT;
-	config.sidSamples = true;
-	config.sampleFormat = IsLittleEndian()
-		? SID2_LITTLE_SIGNED
-		: SID2_BIG_SIGNED;
-#endif
 
-#ifdef HAVE_SIDPLAYFP
-	const bool stereo = tune.getInfo()->sidChips() >= 2;
-#else
-	const bool stereo = tune.isStereo();
-#endif
-
-	if (stereo) {
-#ifdef HAVE_SIDPLAYFP
+	if (tune.getInfo()->sidChips() >= 2) {
 		config.playback = SidConfig::STEREO;
-#else
-		config.playback = sid2_stereo;
-#endif
 		channels = 2;
 	} else {
-#ifdef HAVE_SIDPLAYFP
 		config.playback = SidConfig::MONO;
-#else
-		config.playback = sid2_mono;
-#endif
 		channels = 1;
 	}
 
-#ifdef HAVE_SIDPLAYFP
-	error = !player.config(config);
-#else
-	error = player.config(config) < 0;
-#endif
-	if (error) {
+	if (!player.config(config)) {
 		FmtWarning(sidplay_domain,
 			   "sidplay2.config() failed: {}", player.error());
 		return;
@@ -401,11 +291,7 @@ sidplay_file_decode(DecoderClient &client, Path path_fs)
 
 	/* .. and play */
 
-#ifdef HAVE_SIDPLAYFP
 	constexpr unsigned timebase = 1;
-#else
-	const unsigned timebase = player.timebase();
-#endif
 	const unsigned end = duration.IsNegative()
 		? 0U
 		: duration.ToScale<uint64_t>(timebase);
@@ -418,17 +304,13 @@ sidplay_file_decode(DecoderClient &client, Path path_fs)
 		if (result <= 0)
 			break;
 
-#ifdef HAVE_SIDPLAYFP
 		/* libsidplayfp returns the number of samples */
-		const size_t nbytes = result * sizeof(buffer[0]);
-#else
-		/* libsidplay2 returns the number of bytes */
-		const size_t nbytes = result;
-#endif
+		const size_t n_samples = result;
 
 		client.SubmitTimestamp(FloatDuration(player.time()) / timebase);
 
-		cmd = client.SubmitData(nullptr, buffer, nbytes, 0);
+		cmd = client.SubmitAudio(nullptr, std::span{buffer, n_samples},
+					 0);
 
 		if (cmd == DecoderCommand::SEEK) {
 			unsigned data_time = player.time();
@@ -477,24 +359,18 @@ Windows1252ToUTF8(const char *s) noexcept
 	return t;
 }
 
-gcc_pure
+[[gnu::pure]]
 static AllocatedString
 GetInfoString(const SidTuneInfo &info, unsigned i) noexcept
 {
-#ifdef HAVE_SIDPLAYFP
 	const char *s = info.numberOfInfoStrings() > i
 		? info.infoString(i)
 		: "";
-#else
-	const char *s = info.numberOfInfoStrings > i
-		? info.infoString[i]
-		: "";
-#endif
 
 	return Windows1252ToUTF8(s);
 }
 
-gcc_pure
+[[gnu::pure]]
 static AllocatedString
 GetDateString(const SidTuneInfo &info) noexcept
 {
@@ -529,8 +405,8 @@ ScanSidTuneInfo(const SidTuneInfo &info, unsigned track, unsigned n_tracks,
 
 	if (n_tracks > 1) {
 		const auto tag_title =
-			StringFormat<1024>("%s (%u/%u)",
-					   album.c_str(), track, n_tracks);
+			fmt::format("{} ({}/{})",
+				    album.c_str(), track, n_tracks);
 		handler.OnTag(TAG_TITLE, tag_title.c_str());
 	} else
 		handler.OnTag(TAG_TITLE, album.c_str());
@@ -551,7 +427,7 @@ ScanSidTuneInfo(const SidTuneInfo &info, unsigned track, unsigned n_tracks,
 		handler.OnTag(TAG_DATE, date.c_str());
 
 	/* track */
-	handler.OnTag(TAG_TRACK, StringFormat<16>("%u", track).c_str());
+	handler.OnTag(TAG_TRACK, fmt::format_int{track}.c_str());
 }
 
 static bool
@@ -560,23 +436,15 @@ sidplay_scan_file(Path path_fs, TagHandler &handler) noexcept
 	const auto container = ParseContainerPath(path_fs);
 	const unsigned song_num = container.track;
 
-#ifdef HAVE_SIDPLAYFP
-	SidTune tune(container.path.c_str());
-#else
-	SidTuneMod tune(container.path.c_str());
-#endif
+	auto np = NarrowPath(container.path);
+	auto tune = SidTune(np);
 	if (!tune.getStatus())
 		return false;
 
 	tune.selectSong(song_num);
 
-#ifdef HAVE_SIDPLAYFP
 	const SidTuneInfo &info = *tune.getInfo();
 	const unsigned n_tracks = info.songs();
-#else
-	const SidTuneInfo &info = tune.getInfo();
-	const unsigned n_tracks = info.songs;
-#endif
 
 	ScanSidTuneInfo(info, song_num, n_tracks, handler);
 
@@ -593,21 +461,12 @@ sidplay_container_scan(Path path_fs)
 {
 	std::forward_list<DetachedSong> list;
 
-#ifdef HAVE_SIDPLAYFP
-	SidTune tune(path_fs.c_str());
-#else
-	SidTuneMod tune(path_fs.c_str());
-#endif
+	auto tune = SidTune{NarrowPath(path_fs)};
 	if (!tune.getStatus())
 		return list;
 
-#ifdef HAVE_SIDPLAYFP
 	const SidTuneInfo &info = *tune.getInfo();
 	const unsigned n_tracks = info.songs();
-#else
-	const SidTuneInfo &info = tune.getInfo();
-	const unsigned n_tracks = info.songs;
-#endif
 
 	/* Don't treat sids containing a single tune
 		as containers */
@@ -630,7 +489,7 @@ sidplay_container_scan(Path path_fs)
 		/* Construct container/tune path names, eg.
 		   Delta.sid/tune_001.sid */
 		tail = list.emplace_after(tail,
-					  StringFormat<32>(SUBTUNE_PREFIX "%03u.sid", i),
+					  fmt::format(SUBTUNE_PREFIX "{:03}.sid", i),
 					  tag_builder.Commit());
 	}
 
