@@ -1,33 +1,16 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "BufferedSocket.hxx"
 #include "net/SocketError.hxx"
-#include "util/Compiler.h"
 
 #include <stdexcept>
 
-BufferedSocket::ssize_t
-BufferedSocket::DirectRead(void *data, size_t length) noexcept
+inline BufferedSocket::ssize_t
+BufferedSocket::DirectRead(std::span<std::byte> dest) noexcept
 {
-	const auto nbytes = GetSocket().Read((char *)data, length);
-	if (gcc_likely(nbytes > 0))
+	const auto nbytes = GetSocket().ReadNoWait(dest);
+	if (nbytes > 0) [[likely]]
 		return nbytes;
 
 	if (nbytes == 0) {
@@ -54,7 +37,7 @@ BufferedSocket::ReadToBuffer() noexcept
 	const auto buffer = input.Write();
 	assert(!buffer.empty());
 
-	const auto nbytes = DirectRead(buffer.data, buffer.size);
+	const auto nbytes = DirectRead(buffer);
 	if (nbytes > 0)
 		input.Append(nbytes);
 
@@ -73,7 +56,7 @@ BufferedSocket::ResumeInput() noexcept
 			return true;
 		}
 
-		const auto result = OnSocketInput(buffer.data, buffer.size);
+		const auto result = OnSocketInput(buffer);
 		switch (result) {
 		case InputResult::MORE:
 			if (input.IsFull()) {
@@ -102,7 +85,7 @@ BufferedSocket::OnSocketReady(unsigned flags) noexcept
 {
 	assert(IsDefined());
 
-	if (gcc_unlikely(flags & (SocketEvent::ERROR|SocketEvent::HANGUP))) {
+	if (flags & (SocketEvent::ERROR|SocketEvent::HANGUP)) [[unlikely]] {
 		OnSocketClosed();
 		return;
 	}
