@@ -21,6 +21,29 @@
 
 #include <sys/eventfd.h>
 
+class EventLoop::UringPoll final : Uring::Operation {
+	EventLoop &event_loop;
+
+public:
+	UringPoll(EventLoop &_event_loop) noexcept
+		:event_loop(_event_loop) {}
+
+	void Start();
+
+private:
+	void OnUringCompletion(int res) noexcept override {
+		(void)res; // TODO
+
+		event_loop.epoll_ready = true;
+
+		if (!IsUringPending()) [[unlikely]]
+			/* for some reason, the kernel has stopped our
+			   poll operation (no IORING_CQE_F_MORE):
+			   restart the poll */
+			Start();
+	}
+};
+
 /**
  * Read from the eventfd using io_uring and invoke
  * EventLoop::OnWake().
@@ -104,29 +127,6 @@ EventLoop::SetVolatile() noexcept
 
 #ifdef HAVE_URING
 
-class EventLoop::UringPoll final : Uring::Operation {
-	EventLoop &event_loop;
-
-public:
-	UringPoll(EventLoop &_event_loop) noexcept
-		:event_loop(_event_loop) {}
-
-	void Start();
-
-private:
-	void OnUringCompletion(int res) noexcept override {
-		(void)res; // TODO
-
-		event_loop.epoll_ready = true;
-
-		if (!IsUringPending()) [[unlikely]]
-			/* for some reason, the kernel has stopped our
-			   poll operation (no IORING_CQE_F_MORE):
-			   restart the poll */
-			Start();
-	}
-};
-
 inline void
 EventLoop::UringPoll::Start()
 {
@@ -159,11 +159,11 @@ EventLoop::EnableUring(unsigned entries, struct io_uring_params &params)
 void
 EventLoop::DisableUring() noexcept
 {
-	uring.reset();
 #if defined(HAVE_THREADED_EVENT_LOOP) && defined(USE_EVENTFD)
 	uring_wake.reset();
 #endif
 	uring_poll.reset();
+	uring.reset();
 }
 
 Uring::Queue *
@@ -356,8 +356,9 @@ ExportTimeoutKernelTimespec(Event::Duration timeout, struct __kernel_timespec &b
 		return nullptr;
 
 	if (timeout >= std::chrono::duration_cast<Event::Duration>(std::chrono::hours{24})) [[unlikely]] {
+		using tv_sec_t = std::decay_t<decltype(buffer.tv_sec)>;
 		buffer = {
-			.tv_sec = std::chrono::ceil<std::chrono::duration<__kernel_time64_t>>(timeout).count(),
+			.tv_sec = std::chrono::ceil<std::chrono::duration<tv_sec_t>>(timeout).count(),
 		};
 		return &buffer;
 	}
