@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SmbclientNeighborPlugin.hxx"
 #include "lib/smbclient/Init.hxx"
@@ -99,7 +83,7 @@ void
 SmbclientNeighborExplorer::Close() noexcept
 {
 	{
-		const std::scoped_lock<Mutex> lock(mutex);
+		const std::scoped_lock lock{mutex};
 		quit = true;
 		cond.notify_one();
 	}
@@ -110,7 +94,7 @@ SmbclientNeighborExplorer::Close() noexcept
 NeighborExplorer::List
 SmbclientNeighborExplorer::GetList() const noexcept
 {
-	const std::scoped_lock<Mutex> protect(mutex);
+	const std::scoped_lock protect{mutex};
 	return list;
 }
 
@@ -167,7 +151,7 @@ ReadServers(SmbclientContext &ctx, const char *uri,
 		ReadServers(ctx, handle, list);
 		ctx.CloseDirectory(handle);
 	} else
-		FmtError(smbclient_domain, "smbc_opendir('{}') failed: {}",
+		FmtError(smbclient_domain, "smbc_opendir({:?}) failed: {}",
 			 uri, strerror(errno));
 }
 
@@ -240,7 +224,7 @@ SmbclientNeighborExplorer::ThreadFunc() noexcept
 {
 	SetThreadName("smbclient");
 
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 
 	while (!quit) {
 		Run();
