@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "Registry.hxx"
@@ -25,14 +9,15 @@
 #include "plugins/AoOutputPlugin.hxx"
 #include "plugins/FifoOutputPlugin.hxx"
 #include "plugins/SndioOutputPlugin.hxx"
+#include "plugins/snapcast/SnapcastOutputPlugin.hxx"
 #include "plugins/httpd/HttpdOutputPlugin.hxx"
-#include "plugins/HaikuOutputPlugin.hxx"
 #include "plugins/JackOutputPlugin.hxx"
 #include "plugins/NullOutputPlugin.hxx"
 #include "plugins/OpenALOutputPlugin.hxx"
 #include "plugins/OssOutputPlugin.hxx"
 #include "plugins/OSXOutputPlugin.hxx"
 #include "plugins/PipeOutputPlugin.hxx"
+#include "plugins/PipeWireOutputPlugin.hxx"
 #include "plugins/PulseOutputPlugin.hxx"
 #include "plugins/RecorderOutputPlugin.hxx"
 #include "plugins/ShoutOutputPlugin.hxx"
@@ -42,11 +27,11 @@
 #include "plugins/WinmmOutputPlugin.hxx"
 #endif
 #ifdef ENABLE_WASAPI_OUTPUT
-#include "plugins/WasapiOutputPlugin.hxx"
+#include "plugins/wasapi/WasapiOutputPlugin.hxx"
 #endif
 #include "util/StringAPI.hxx"
 
-const AudioOutputPlugin *const audio_output_plugins[] = {
+constinit const AudioOutputPlugin *const audio_output_plugins[] = {
 #ifdef HAVE_SHOUT
 	&shout_output_plugin,
 #endif
@@ -60,9 +45,6 @@ const AudioOutputPlugin *const audio_output_plugins[] = {
 #ifdef ENABLE_SNDIO
 	&sndio_output_plugin,
 #endif
-#ifdef ENABLE_HAIKU
-	&haiku_output_plugin,
-#endif
 #ifdef ENABLE_PIPE_OUTPUT
 	&pipe_output_plugin,
 #endif
@@ -84,6 +66,9 @@ const AudioOutputPlugin *const audio_output_plugins[] = {
 #ifdef ENABLE_SOLARIS_OUTPUT
 	&solaris_output_plugin,
 #endif
+#ifdef ENABLE_PIPEWIRE
+	&pipewire_output_plugin,
+#endif
 #ifdef ENABLE_PULSE
 	&pulse_output_plugin,
 #endif
@@ -93,6 +78,9 @@ const AudioOutputPlugin *const audio_output_plugins[] = {
 #ifdef ENABLE_HTTPD_OUTPUT
 	&httpd_output_plugin,
 #endif
+#ifdef ENABLE_SNAPCAST_OUTPUT
+	&snapcast_output_plugin,
+#endif
 #ifdef ENABLE_RECORDER_OUTPUT
 	&recorder_output_plugin,
 #endif
@@ -106,11 +94,12 @@ const AudioOutputPlugin *const audio_output_plugins[] = {
 };
 
 const AudioOutputPlugin *
-AudioOutputPlugin_get(const char *name)
+GetAudioOutputPluginByName(const char *name) noexcept
 {
-	audio_output_plugins_for_each(plugin)
-		if (StringIsEqual(plugin->name, name))
-			return plugin;
+	for (const auto &plugin : GetAllAudioOutputPlugins()) {
+		if (StringIsEqual(plugin.name, name))
+			return &plugin;
+	}
 
 	return nullptr;
 }
