@@ -1,34 +1,16 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Export.hxx"
 #include "Order.hxx"
 #include "Pack.hxx"
 #include "Silence.hxx"
 #include "util/ByteReverse.hxx"
-#include "util/ConstBuffer.hxx"
-#include "util/WritableBuffer.hxx"
+#include "util/SpanCast.hxx"
 
+#include <algorithm>
 #include <cassert>
 
-#include <string.h>
-
 void
 PcmExport::Open(SampleFormat sample_format, unsigned _channels,
 		Params params) noexcept
@@ -94,14 +76,14 @@ PcmExport::Open(SampleFormat sample_format, unsigned _channels,
 	}
 
 	/* prepare a moment of silence for GetSilence() */
-	char buffer[sizeof(silence_buffer)];
+	std::byte buffer[sizeof(silence_buffer)];
 	const size_t buffer_size = GetInputBlockSize();
 	assert(buffer_size < sizeof(buffer));
 	PcmSilence({buffer, buffer_size}, src_sample_format);
 	auto s = Export({buffer, buffer_size});
-	assert(s.size < sizeof(silence_buffer));
-	silence_size = s.size;
-	memcpy(silence_buffer, s.data, s.size);
+	assert(s.size() < sizeof(silence_buffer));
+	silence_size = s.size();
+	std::copy(s.begin(), s.end(), silence_buffer);
 }
 
 void
@@ -201,7 +183,7 @@ PcmExport::GetOutputBlockSize() const noexcept
 	return GetOutputFrameSize();
 }
 
-ConstBuffer<void>
+std::span<const std::byte>
 PcmExport::GetSilence() const noexcept
 {
 	return {silence_buffer, silence_size};
@@ -263,8 +245,8 @@ PcmExport::Params::CalcInputSampleRate(unsigned sample_rate) const noexcept
 	return sample_rate;
 }
 
-ConstBuffer<void>
-PcmExport::Export(ConstBuffer<void> data) noexcept
+std::span<const std::byte>
+PcmExport::Export(std::span<const std::byte> data) noexcept
 {
 	if (alsa_channel_order)
 		data = ToAlsaChannelOrder(order_buffer, data,
@@ -276,38 +258,34 @@ PcmExport::Export(ConstBuffer<void> data) noexcept
 		break;
 
 	case DsdMode::U16:
-		data = dsd16_converter.Convert(ConstBuffer<uint8_t>::FromVoid(data))
-			.ToVoid();
+		data = std::as_bytes(dsd16_converter.Convert(data));
 		break;
 
 	case DsdMode::U32:
-		data = dsd32_converter.Convert(ConstBuffer<uint8_t>::FromVoid(data))
-			.ToVoid();
+		data = std::as_bytes(dsd32_converter.Convert(data));
 		break;
 
 	case DsdMode::DOP:
-		data = dop_converter.Convert(ConstBuffer<uint8_t>::FromVoid(data))
-			.ToVoid();
+		data = std::as_bytes(dop_converter.Convert(data));
 		break;
 	}
 #endif
 
 	if (pack24) {
-		const auto src = ConstBuffer<int32_t>::FromVoid(data);
-		const size_t num_samples = src.size;
+		const auto src = FromBytesStrict<const int32_t>(data);
+		const size_t num_samples = src.size();
 		const size_t dest_size = num_samples * 3;
 		auto *dest = (uint8_t *)pack_buffer.Get(dest_size);
 		assert(dest != nullptr);
 
-		pcm_pack_24(dest, src.begin(), src.end());
+		pcm_pack_24(dest, src.data(), src.data() + src.size());
 
-		data.data = dest;
-		data.size = dest_size;
+		data = std::as_bytes(std::span{dest, dest_size});
 	} else if (shift8) {
-		const auto src = ConstBuffer<int32_t>::FromVoid(data);
+		const auto src = FromBytesStrict<const int32_t>(data);
 
-		auto *dest = (uint32_t *)pack_buffer.Get(data.size);
-		data.data = dest;
+		auto *dest = (uint32_t *)pack_buffer.Get(data.size());
+		data = {(const std::byte *)dest, data.size()};
 
 		for (auto i : src)
 			*dest++ = i << 8;
@@ -316,13 +294,14 @@ PcmExport::Export(ConstBuffer<void> data) noexcept
 	if (reverse_endian > 0) {
 		assert(reverse_endian >= 2);
 
-		const auto src = ConstBuffer<uint8_t>::FromVoid(data);
+		const auto src = FromBytesStrict<const uint8_t>(data);
 
-		auto *dest = (uint8_t *)reverse_buffer.Get(data.size);
+		auto *dest = (uint8_t *)reverse_buffer.Get(data.size());
 		assert(dest != nullptr);
-		data.data = dest;
+		data = {(const std::byte *)dest, data.size()};
 
-		reverse_bytes(dest, src.begin(), src.end(), reverse_endian);
+		reverse_bytes(dest, src.data(), src.data() + src.size(),
+			      reverse_endian);
 	}
 
 	return data;
