@@ -1,28 +1,11 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Volume.hxx"
 #include "Silence.hxx"
 #include "Traits.hxx"
-#include "util/ConstBuffer.hxx"
-#include "util/WritableBuffer.hxx"
-#include "util/RuntimeError.hxx"
+#include "lib/fmt/AudioFormatFormatter.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "util/TransformN.hxx"
 
 #include "Dither.cxx" // including the .cxx file to get inlined templates
@@ -32,22 +15,13 @@
 
 #include <string.h>
 
-#if GCC_OLDER_THAN(8,0)
-/* GCC 6.3 emits this bogus warning in PcmVolumeConvert() because it
-   checks an unreachable branch */
-#pragma GCC diagnostic ignored "-Wshift-count-overflow"
-#endif
-
 /**
  * Apply software volume, converting to a different sample type.
  */
 template<SampleFormat SF, SampleFormat DF,
 	 class STraits=SampleTraits<SF>,
 	 class DTraits=SampleTraits<DF>>
-#if !GCC_OLDER_THAN(8,0)
-constexpr
-#endif
-static typename DTraits::value_type
+static constexpr typename DTraits::value_type
 PcmVolumeConvert(typename STraits::value_type _sample, int volume) noexcept
 {
 	typename STraits::long_type sample(_sample);
@@ -165,8 +139,8 @@ PcmVolume::Open(SampleFormat _format, bool allow_convert)
 
 	switch (_format) {
 	case SampleFormat::UNDEFINED:
-		throw FormatRuntimeError("Software volume for %s is not implemented",
-					 sample_format_to_string(_format));
+		throw FmtRuntimeError("Software volume for {} is not implemented",
+				      _format);
 
 	case SampleFormat::S8:
 		break;
@@ -195,13 +169,13 @@ PcmVolume::Open(SampleFormat _format, bool allow_convert)
 	return format = _format;
 }
 
-ConstBuffer<void>
-PcmVolume::Apply(ConstBuffer<void> src) noexcept
+std::span<const std::byte>
+PcmVolume::Apply(std::span<const std::byte> src) noexcept
 {
 	if (volume == PCM_VOLUME_1 && !convert)
 		return src;
 
-	size_t dest_size = src.size;
+	size_t dest_size = src.size();
 	if (convert) {
 		assert(format == SampleFormat::S16);
 
@@ -213,8 +187,9 @@ PcmVolume::Apply(ConstBuffer<void> src) noexcept
 
 	if (volume == 0) {
 		/* optimized special case: 0% volume = memset(0) */
-		PcmSilence({data, dest_size}, format);
-		return { data, dest_size };
+		PcmSilence(std::span{(std::byte *)data, dest_size},
+			   format);
+		return { (const std::byte *)data, dest_size };
 	}
 
 	switch (format) {
@@ -224,42 +199,42 @@ PcmVolume::Apply(ConstBuffer<void> src) noexcept
 
 	case SampleFormat::S8:
 		pcm_volume_change_8(dither, (int8_t *)data,
-				    (const int8_t *)src.data,
-				    src.size / sizeof(int8_t),
+				    (const int8_t *)src.data(),
+				    src.size() / sizeof(int8_t),
 				    volume);
 		break;
 
 	case SampleFormat::S16:
 		if (convert)
 			PcmVolumeChange16to32((int32_t *)data,
-					      (const int16_t *)src.data,
-					      src.size / sizeof(int16_t),
+					      (const int16_t *)src.data(),
+					      src.size() / sizeof(int16_t),
 					      volume);
 		else
 			pcm_volume_change_16(dither, (int16_t *)data,
-					     (const int16_t *)src.data,
-					     src.size / sizeof(int16_t),
+					     (const int16_t *)src.data(),
+					     src.size() / sizeof(int16_t),
 					     volume);
 		break;
 
 	case SampleFormat::S24_P32:
 		pcm_volume_change_24(dither, (int32_t *)data,
-				     (const int32_t *)src.data,
-				     src.size / sizeof(int32_t),
+				     (const int32_t *)src.data(),
+				     src.size() / sizeof(int32_t),
 				     volume);
 		break;
 
 	case SampleFormat::S32:
 		pcm_volume_change_32(dither, (int32_t *)data,
-				     (const int32_t *)src.data,
-				     src.size / sizeof(int32_t),
+				     (const int32_t *)src.data(),
+				     src.size() / sizeof(int32_t),
 				     volume);
 		break;
 
 	case SampleFormat::FLOAT:
 		pcm_volume_change_float((float *)data,
-					(const float *)src.data,
-					src.size / sizeof(float),
+					(const float *)src.data(),
+					src.size() / sizeof(float),
 					pcm_volume_to_float(volume));
 		break;
 
@@ -268,5 +243,5 @@ PcmVolume::Apply(ConstBuffer<void> src) noexcept
 		return src;
 	}
 
-	return { data, dest_size };
+	return { (const std::byte *)data, dest_size };
 }
