@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 /** \file
  *
@@ -28,9 +12,9 @@
 #include "../MemorySongEnumerator.hxx"
 #include "lib/xiph/FlacMetadataChain.hxx"
 #include "lib/xiph/FlacMetadataIterator.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "song/DetachedSong.hxx"
 #include "input/InputStream.hxx"
-#include "util/RuntimeError.hxx"
 
 #include <FLAC/metadata.h>
 
@@ -67,8 +51,8 @@ flac_playlist_open_stream(InputStreamPtr &&is)
 {
 	FlacMetadataChain chain;
 	if (!chain.Read(*is))
-		throw FormatRuntimeError("Failed to read FLAC metadata: %s",
-					 chain.GetStatusString());
+		throw FmtRuntimeError("Failed to read FLAC metadata: {}",
+				      chain.GetStatusString());
 
 	FlacMetadataIterator iterator((FLAC__Metadata_Chain *)chain);
 
