@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "SoundCloudPlaylistPlugin.hxx"
 #include "../PlaylistPlugin.hxx"
@@ -26,11 +10,10 @@
 #include "config/Block.hxx"
 #include "input/InputStream.hxx"
 #include "tag/Builder.hxx"
+#include "util/AllocatedString.hxx"
 #include "util/ASCII.hxx"
 #include "util/StringCompare.hxx"
-#include "util/Alloc.hxx"
 #include "util/Domain.hxx"
-#include "util/ScopeExit.hxx"
 #include "Log.hxx"
 
 #include <string>
@@ -38,6 +21,8 @@
 #include <string.h>
 #include <stdlib.h>
 
+using std::string_view_literals::operator""sv;
+
 static struct {
 	std::string apikey;
 } soundcloud_config;
@@ -64,26 +49,59 @@ soundcloud_init(const ConfigBlock &block)
  * @param uri uri of a soundcloud page (or just the path)
  * @return Constructed URL. Must be freed with free().
  */
-static char *
-soundcloud_resolve(const char* uri)
+static AllocatedString
+soundcloud_resolve(std::string_view uri) noexcept
 {
-	char *u, *ru;
-
-	if (StringStartsWithCaseASCII(uri, "https://")) {
-		u = xstrdup(uri);
-	} else if (StringStartsWith(uri, "soundcloud.com")) {
-		u = xstrcatdup("https://", uri);
-	} else {
-		/* assume it's just a path on soundcloud.com */
-		u = xstrcatdup("https://soundcloud.com/", uri);
+	if (StringStartsWithIgnoreCase(uri, "https://"sv)) {
+		return AllocatedString{uri};
+	} else if (uri.starts_with("soundcloud.com"sv)) {
+		return AllocatedString{"https://"sv, uri};
 	}
 
-	ru = xstrcatdup("https://api.soundcloud.com/resolve.json?url=",
-			u, "&client_id=",
-			soundcloud_config.apikey.c_str());
-	free(u);
 
-	return ru;
+	/* assume it's just a path on soundcloud.com */
+	AllocatedString u{"https://soundcloud.com/"sv, uri};
+
+	return AllocatedString{
+		"https://api.soundcloud.com/resolve.json?url="sv,
+		u, "&client_id="sv,
+		soundcloud_config.apikey,
+	};
+}
+
+static AllocatedString
+TranslateSoundCloudUri(std::string_view uri) noexcept
+{
+	if (SkipPrefix(uri, "track/"sv)) {
+		return AllocatedString{
+			"https://api.soundcloud.com/tracks/"sv,
+			uri, ".json?client_id="sv,
+			soundcloud_config.apikey,
+		};
+	} else if (SkipPrefix(uri, "playlist/"sv)) {
+		return AllocatedString{
+			"https://api.soundcloud.com/playlists/"sv,
+			uri, ".json?client_id="sv,
+			soundcloud_config.apikey,
+		};
+	} else if (SkipPrefix(uri, "user/"sv)) {
+		return AllocatedString{
+			"https://api.soundcloud.com/users/"sv,
+			uri, "/tracks.json?client_id="sv,
+			soundcloud_config.apikey,
+		};
+	} else if (SkipPrefix(uri, "search/"sv)) {
+		return AllocatedString{
+			"https://api.soundcloud.com/tracks.json?q="sv,
+			uri, "&client_id="sv,
+			soundcloud_config.apikey,
+		};
+	} else if (SkipPrefix(uri, "url/"sv)) {
+		/* Translate to soundcloud resolver call. libcurl will automatically
+		   follow the redirect to the right resource. */
+		return soundcloud_resolve(uri);
+	} else
+		return nullptr;
 }
 
 /* YAJL parser for track data from both /tracks/ and /playlists/ JSON */
@@ -112,9 +130,9 @@ struct SoundCloudJsonData {
 	std::forward_list<DetachedSong> songs;
 
 	bool Integer(long long value) noexcept;
-	bool String(StringView value) noexcept;
+	bool String(std::string_view value) noexcept;
 	bool StartMap() noexcept;
-	bool MapKey(StringView value) noexcept;
+	bool MapKey(std::string_view value) noexcept;
 	bool EndMap() noexcept;
 };
 
@@ -133,15 +151,15 @@ SoundCloudJsonData::Integer(long long intval) noexcept
 }
 
 inline bool
-SoundCloudJsonData::String(StringView value) noexcept
+SoundCloudJsonData::String(std::string_view value) noexcept
 {
 	switch (key) {
 	case SoundCloudJsonData::Key::TITLE:
-		title.assign(value.data, value.size);
+		title = value;
 		break;
 
 	case SoundCloudJsonData::Key::STREAM_URL:
-		stream_url.assign(value.data, value.size);
+		stream_url = value;
 		got_url = 1;
 		break;
 
@@ -153,7 +171,7 @@ SoundCloudJsonData::String(StringView value) noexcept
 }
 
 inline bool
-SoundCloudJsonData::MapKey(StringView value) noexcept
+SoundCloudJsonData::MapKey(std::string_view value) noexcept
 {
 	const auto *i = key_str;
 	while (*i != nullptr && !StringStartsWith(*i, value))
@@ -192,7 +210,7 @@ SoundCloudJsonData::EndMap() noexcept
 	TagBuilder tag;
 	tag.SetDuration(SignedSongTime::FromMS(duration));
 	if (!title.empty())
-		tag.AddItem(TAG_NAME, title.c_str());
+		tag.AddItem(TAG_NAME, title);
 
 	songs.emplace_front(u.c_str(), tag.Commit());
 
@@ -240,36 +258,7 @@ soundcloud_open_uri(const char *uri, Mutex &mutex)
 	assert(StringEqualsCaseASCII(uri, "soundcloud://", 13));
 	uri += 13;
 
-	char *u = nullptr;
-	if (strncmp(uri, "track/", 6) == 0) {
-		const char *rest = uri + 6;
-		u = xstrcatdup("https://api.soundcloud.com/tracks/",
-			       rest, ".json?client_id=",
-			       soundcloud_config.apikey.c_str());
-	} else if (strncmp(uri, "playlist/", 9) == 0) {
-		const char *rest = uri + 9;
-		u = xstrcatdup("https://api.soundcloud.com/playlists/",
-			       rest, ".json?client_id=",
-			       soundcloud_config.apikey.c_str());
-	} else if (strncmp(uri, "user/", 5) == 0) {
-		const char *rest = uri + 5;
-		u = xstrcatdup("https://api.soundcloud.com/users/",
-			       rest, "/tracks.json?client_id=",
-			       soundcloud_config.apikey.c_str());
-	} else if (strncmp(uri, "search/", 7) == 0) {
-		const char *rest = uri + 7;
-		u = xstrcatdup("https://api.soundcloud.com/tracks.json?q=",
-			       rest, "&client_id=",
-			       soundcloud_config.apikey.c_str());
-	} else if (strncmp(uri, "url/", 4) == 0) {
-		const char *rest = uri + 4;
-		/* Translate to soundcloud resolver call. libcurl will automatically
-		   follow the redirect to the right resource. */
-		u = soundcloud_resolve(rest);
-	}
-
-	AtScopeExit(u) { free(u); };
-
+	auto u = TranslateSoundCloudUri(uri);
 	if (u == nullptr) {
 		LogWarning(soundcloud_domain, "unknown soundcloud URI");
 		return nullptr;
@@ -277,7 +266,7 @@ soundcloud_open_uri(const char *uri, Mutex &mutex)
 
 	SoundCloudJsonData data;
 	Yajl::Handle handle(&parse_callbacks, nullptr, &data);
-	soundcloud_parse_json(u, handle, mutex);
+	soundcloud_parse_json(u.c_str(), handle, mutex);
 
 	data.songs.reverse();
 	return std::make_unique<MemorySongEnumerator>(std::move(data.songs));
