@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2020 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "tag/ApeLoader.hxx"
@@ -24,7 +8,6 @@
 #include "fs/NarrowPath.hxx"
 #include "input/InputStream.hxx"
 #include "input/LocalOpen.hxx"
-#include "util/StringView.hxx"
 #include "util/PrintException.hxx"
 
 #include <stdlib.h>
@@ -36,11 +19,12 @@
 
 static bool
 MyApeTagCallback([[maybe_unused]] unsigned long flags,
-		 const char *key, StringView value)
+		 const char *key, std::string_view value)
 {
 	if ((flags & (0x3 << 1)) == 0)
 		// UTF-8
-		printf("\"%s\"=\"%.*s\"\n", key, (int)value.size, value.data);
+		printf("\"%s\"=\"%.*s\"\n", key,
+		       (int)value.size(), value.data());
 	else
 		printf("\"%s\"=0x%lx\n", key, flags);
 	return true;
