@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "config/Data.hxx"
@@ -26,12 +10,12 @@
 #include "input/InputStream.hxx"
 #include "tag/Handler.hxx"
 #include "tag/Generic.hxx"
+#include "tag/Names.hxx"
 #include "fs/Path.hxx"
 #include "fs/NarrowPath.hxx"
 #include "pcm/AudioFormat.hxx"
 #include "util/ScopeExit.hxx"
 #include "util/StringBuffer.hxx"
-#include "util/StringView.hxx"
 #include "util/PrintException.hxx"
 
 #include <cassert>
@@ -60,16 +44,16 @@ public:
 		printf("duration=%f\n", duration.ToDoubleS());
 	}
 
-	void OnTag(TagType type, StringView value) noexcept override {
+	void OnTag(TagType type, std::string_view value) noexcept override {
 		printf("[%s]=%.*s\n", tag_item_names[type],
-		       int(value.size), value.data);
+		       int(value.size()), value.data());
 		empty = false;
 	}
 
-	void OnPair(StringView key, StringView value) noexcept override {
+	void OnPair(std::string_view key, std::string_view value) noexcept override {
 		printf("\"%.*s\"=%.*s\n",
-		       int(key.size), key.data,
-		       int(value.size), value.data);
+		       int(key.size()), key.data(),
+		       int(value.size()), value.data());
 	}
 
 	void OnAudioFormat(AudioFormat af) noexcept override {
@@ -77,8 +61,9 @@ public:
 	}
 
 	void OnPicture(const char *mime_type,
-		       ConstBuffer<void> buffer) noexcept override {
-		printf("picture mime='%s' size=%zu\n", mime_type, buffer.size);
+		       std::span<const std::byte> buffer) noexcept override {
+		printf("picture mime='%s' size=%zu\n",
+		       mime_type, buffer.size());
 	}
 };
 
